@@ -14,8 +14,8 @@
  *  limitations under the License.
  */
 
-#ifndef _ONEDPL_GCD_IMPL
-#define _ONEDPL_GCD_IMPL
+#ifndef _ONEDPL_GCD_IMPL_H
+#define _ONEDPL_GCD_IMPL_H
 
 #include <limits>
 #include <type_traits>
@@ -36,14 +36,14 @@ __abs_impl(_Source __t, ::std::true_type)
     if (__t == ::std::numeric_limits<_Source>::min())
         return -static_cast<_Result>(__t);
     return -__t;
-};
+}
 
 template <typename _Result, typename _Source>
 _Result
 __abs_impl(_Source __t, ::std::false_type)
 {
     return __t;
-};
+}
 
 template <typename _Result, typename _Source>
 constexpr _Result
@@ -63,10 +63,9 @@ template <typename _Mn, typename _Nn>
 constexpr ::std::common_type_t<_Mn, _Nn>
 gcd(_Mn __m, _Nn __n)
 {
-    static_assert((::std::is_integral<_Mn>::value && ::std::is_integral<_Nn>::value),
-                  "Arguments to gcd must be integer types");
-    static_assert((!::std::is_same<::std::remove_cv_t<_Mn>, bool>::value), "First argument to gcd cannot be bool");
-    static_assert((!::std::is_same<::std::remove_cv_t<_Nn>, bool>::value), "Second argument to gcd cannot be bool");
+    static_assert((::std::is_integral_v<_Mn> && ::std::is_integral_v<_Nn>), "Arguments to gcd must be integer types");
+    static_assert((!::std::is_same_v<::std::remove_cv_t<_Mn>, bool>), "First argument to gcd cannot be bool");
+    static_assert((!::std::is_same_v<::std::remove_cv_t<_Nn>, bool>), "Second argument to gcd cannot be bool");
     using _Rp = ::std::common_type_t<_Mn, _Nn>;
     using _Wp = ::std::make_unsigned_t<_Rp>;
     _Wp __m1 = static_cast<_Wp>(oneapi::dpl::internal::__get_abs<_Rp>(__m));
@@ -86,10 +85,9 @@ template <typename _Mn, typename _Nn>
 constexpr ::std::common_type_t<_Mn, _Nn>
 lcm(_Mn __m, _Nn __n)
 {
-    static_assert((::std::is_integral<_Mn>::value && ::std::is_integral<_Nn>::value),
-                  "Arguments to lcm must be integer types");
-    static_assert((!::std::is_same<::std::remove_cv_t<_Mn>, bool>::value), "First argument to lcm cannot be bool");
-    static_assert((!::std::is_same<::std::remove_cv_t<_Nn>, bool>::value), "Second argument to lcm cannot be bool");
+    static_assert((::std::is_integral_v<_Mn> && ::std::is_integral_v<_Nn>), "Arguments to lcm must be integer types");
+    static_assert((!::std::is_same_v<::std::remove_cv_t<_Mn>, bool>), "First argument to lcm cannot be bool");
+    static_assert((!::std::is_same_v<::std::remove_cv_t<_Nn>, bool>), "Second argument to lcm cannot be bool");
     if (__m == 0 || __n == 0)
         return 0;
     using _Rp = ::std::common_type_t<_Mn, _Nn>;
@@ -101,4 +99,4 @@ lcm(_Mn __m, _Nn __n)
 } // end namespace dpl
 } // end namespace oneapi
 
-#endif
+#endif // _ONEDPL_GCD_IMPL_H
