@@ -17,7 +17,7 @@
 #ifndef _ONEDPL_SERIAL_NUMERIC_IMPL_H
 #define _ONEDPL_SERIAL_NUMERIC_IMPL_H
 
-#if (__cplusplus >= 201703L)
+#if _ONEDPL___cplusplus >= 201703L
 #    include <functional>
 #    include <iterator>
 #    include <numeric>
@@ -31,15 +31,15 @@ _Tp
 reduce(_InputIterator __first, _InputIterator __last, _Tp __init, _BinaryOp __b)
 {
     for (; __first != __last; ++__first)
-        __init = __b(__init, *__first);
-    return __init;
+        __init = __b(std::move(__init), *__first);
+    return std::move(__init);
 }
 
 template <class _InputIterator, class _Tp>
 _Tp
 reduce(_InputIterator __first, _InputIterator __last, _Tp __init)
 {
-    return oneapi::dpl::reduce(__first, __last, __init, ::std::plus<_Tp>());
+    return oneapi::dpl::reduce(__first, __last, std::move(__init), ::std::plus<_Tp>());
 }
 
 template <class _InputIterator>
@@ -53,5 +53,5 @@ using ::std::reduce;
 #    endif
 } // namespace dpl
 } // namespace oneapi
-#endif // __cplusplus >= 201703L
-#endif /* _ONEDPL_SERIAL_NUMERIC_IMPL_H */
+#endif // _ONEDPL___cplusplus >= 201703L
+#endif // _ONEDPL_SERIAL_NUMERIC_IMPL_H
