@@ -62,7 +62,7 @@ template <class _RandomAccessIterator, class _Function>
 void __brick_walk1(_RandomAccessIterator, _RandomAccessIterator, _Function,
                    /*vector=*/::std::true_type);
 
-template <class _Tag, class _ExecutionPolicy, class _ForwardIterator, class _Function, class _IsVector>
+template <class _Tag, class _ExecutionPolicy, class _ForwardIterator, class _Function>
 void
 __pattern_walk1(_Tag, _ExecutionPolicy&&, _ForwardIterator, _ForwardIterator, _Function);
 
@@ -430,11 +430,17 @@ __brick_swap_ranges(_RandomAccessIterator __first, _RandomAccessIterator __last,
 
 template <class _ForwardIterator, class _OutputIterator, class _UnaryPredicate>
 _OutputIterator __brick_copy_if(_ForwardIterator, _ForwardIterator, _OutputIterator, _UnaryPredicate,
-                                /*vector=*/::std::false_type) noexcept;
+                                /*vector=*/std::false_type) noexcept;
 
 template <class _RandomAccessIterator, class _OutputIterator, class _UnaryPredicate>
 _OutputIterator __brick_copy_if(_RandomAccessIterator, _RandomAccessIterator, _OutputIterator, _UnaryPredicate,
-                                /*vector=*/::std::true_type) noexcept;
+                                /*vector=*/std::true_type) noexcept;
+
+template <class _RandomAccessIterator1, class _RandomAccessIterator2, class _UnaryPredicate>
+std::pair<_RandomAccessIterator1, _RandomAccessIterator2>
+__brick_bounded_copy_if(_RandomAccessIterator1, typename std::iterator_traits<_RandomAccessIterator1>::difference_type,
+                        _RandomAccessIterator2, typename std::iterator_traits<_RandomAccessIterator2>::difference_type,
+                        _UnaryPredicate, /*vector=*/std::true_type) noexcept;
 
 template <class _DifferenceType, class _ForwardIterator, class _UnaryPredicate>
 ::std::pair<_DifferenceType, _DifferenceType>
@@ -455,6 +461,16 @@ void
 __brick_copy_by_mask(_RandomAccessIterator, _RandomAccessIterator, _OutputIterator, bool* __restrict,
                      /*vector=*/::std::true_type) noexcept;
 
+template <class _RandomAccessIterator1, class _RandomAccessIterator2, class _Bound, class _Assigner>
+_Bound
+__brick_bounded_copy_by_mask(_RandomAccessIterator1, _Bound, _RandomAccessIterator2, _Bound, bool*, _Assigner,
+                             /*vector=*/std::false_type) noexcept;
+
+template <class _RandomAccessIterator1, class _RandomAccessIterator2, class _Bound, class _Assigner>
+_Bound
+__brick_bounded_copy_by_mask(_RandomAccessIterator1, _Bound, _RandomAccessIterator2, _Bound, bool*, _Assigner,
+                             /*vector=*/std::true_type) noexcept;
+
 template <class _ForwardIterator, class _OutputIterator1, class _OutputIterator2>
 void
 __brick_partition_by_mask(_ForwardIterator, _ForwardIterator, _OutputIterator1, _OutputIterator2, bool*,
@@ -476,6 +492,14 @@ _RandomAccessIterator2
 __pattern_copy_if(__parallel_tag<_IsVector>, _ExecutionPolicy&&, _RandomAccessIterator1, _RandomAccessIterator1,
                   _RandomAccessIterator2, _UnaryPredicate);
 
+template <class _IsVector, class _ExecutionPolicy, class _RandomAccessIterator1, class _RandomAccessIterator2,
+          class _UnaryPredicate>
+std::pair<_RandomAccessIterator1, _RandomAccessIterator2>
+__pattern_bounded_copy_if(__parallel_tag<_IsVector>, _ExecutionPolicy&&, _RandomAccessIterator1,
+                          typename std::iterator_traits<_RandomAccessIterator1>::difference_type,
+                          _RandomAccessIterator2,
+                          typename std::iterator_traits<_RandomAccessIterator2>::difference_type, _UnaryPredicate);
+
 //------------------------------------------------------------------------
 // count
 //------------------------------------------------------------------------
@@ -1221,9 +1245,9 @@ __pattern_lexicographical_compare(__parallel_tag<_IsVector>, _ExecutionPolicy&&,
                                   _RandomAccessIterator1, _RandomAccessIterator2, _RandomAccessIterator2,
                                   _Compare) noexcept;
 
-template <class _Tag, class _ExecutionPolicy, class _ForwardIterator1, class _ForwardIterator2, class _Function>
+template <class _Tag, class _ExecutionPolicy, class _ForwardIterator1, class _ForwardIterator2>
 _ForwardIterator2
-__pattern_swap(_Tag, _ExecutionPolicy&&, _ForwardIterator1, _ForwardIterator1, _ForwardIterator2, _Function);
+__pattern_swap(_Tag, _ExecutionPolicy&&, _ForwardIterator1, _ForwardIterator1, _ForwardIterator2);
 
 //------------------------------------------------------------------------
 // shift_left
