@@ -18,6 +18,8 @@
 
 #include "sycl_traits.h" //SYCL traits specialization for some oneDPL types.
 
+#include <functional> // for std::invoke
+
 //The file is an internal file and the code of that file is included by a major file into the following namespaces:
 //namespace oneapi
 //{
@@ -37,18 +39,23 @@ struct __subgroup_radix_sort
     sycl::event
     operator()(sycl::queue& __q, _RangeIn&& __src, _Proj __proj)
     {
-        using __wg_size_t = ::std::integral_constant<::std::uint16_t, __wg_size>;
-        using __block_size_t = ::std::integral_constant<::std::uint16_t, __block_size>;
-        using __call_0_t = ::std::integral_constant<::std::uint16_t, 0>;
-        using __call_1_t = ::std::integral_constant<::std::uint16_t, 1>;
-        using __call_2_t = ::std::integral_constant<::std::uint16_t, 2>;
+        using __wg_size_t = std::integral_constant<::std::uint16_t, __wg_size>;
+        using __block_size_t = std::integral_constant<::std::uint16_t, __block_size>;
+        using __radix_t = std::integral_constant<::std::uint32_t, __radix>;
+        using __call_0_t = std::integral_constant<::std::uint16_t, 0>;
+        using __call_1_t = std::integral_constant<::std::uint16_t, 1>;
+        using __call_2_t = std::integral_constant<::std::uint16_t, 2>;
+        using __is_asc_t = std::integral_constant<bool, __is_asc>;
 
-        using _SortKernelLoc = oneapi::dpl::__par_backend_hetero::__internal::__kernel_name_provider<
-            __radix_sort_one_wg_kernel<_KernelNameBase, __wg_size_t, __block_size_t, __call_0_t>>;
-        using _SortKernelPartGlob = oneapi::dpl::__par_backend_hetero::__internal::__kernel_name_provider<
-            __radix_sort_one_wg_kernel<_KernelNameBase, __wg_size_t, __block_size_t, __call_1_t>>;
-        using _SortKernelGlob = oneapi::dpl::__par_backend_hetero::__internal::__kernel_name_provider<
-            __radix_sort_one_wg_kernel<_KernelNameBase, __wg_size_t, __block_size_t, __call_2_t>>;
+        using _SortKernelLoc =
+            oneapi::dpl::__par_backend_hetero::__internal::__kernel_name_provider<__radix_sort_one_wg_kernel<
+                _KernelNameBase, __wg_size_t, __block_size_t, __radix_t, __call_0_t, __is_asc_t>>;
+        using _SortKernelPartGlob =
+            oneapi::dpl::__par_backend_hetero::__internal::__kernel_name_provider<__radix_sort_one_wg_kernel<
+                _KernelNameBase, __wg_size_t, __block_size_t, __radix_t, __call_1_t, __is_asc_t>>;
+        using _SortKernelGlob =
+            oneapi::dpl::__par_backend_hetero::__internal::__kernel_name_provider<__radix_sort_one_wg_kernel<
+                _KernelNameBase, __wg_size_t, __block_size_t, __radix_t, __call_2_t, __is_asc_t>>;
 
         using _KeyT = oneapi::dpl::__internal::__value_t<_RangeIn>;
         //check SLM size
@@ -209,9 +216,13 @@ struct __subgroup_radix_sort
                                 for (uint16_t __i = 0; __i < __block_size; ++__i)
                                 {
                                     const uint16_t __idx = __wi * __block_size + __i;
-                                    const uint16_t __bin = __idx < __n ? __get_bucket</*mask*/ __bin_count - 1>(
-                                        __order_preserving_cast<__is_asc>(__proj(__values.__v[__i])), __begin_bit)
-                                        : __bin_count - 1/*default bin for out of range elements (when idx >= n)*/;
+                                    const uint16_t __bin =
+                                        __idx < __n
+                                            ? __get_bucket</*mask*/ __bin_count - 1>(
+                                                  __order_preserving_cast<__is_asc>(
+                                                      std::invoke(__proj, __values.__v[__i])),
+                                                  __begin_bit)
+                                            : __bin_count - 1 /*default bin for out of range elements (when idx >= n)*/;
 
                                     //"counting" and local offset calculation
                                     __counters[__i] = &__pcounter[__bin * __wg_size];
