@@ -22,6 +22,8 @@
 #include <type_traits>
 #include <tuple>
 #include <algorithm>
+#include <optional>
+#include <cassert>
 
 #include "../../iterator_impl.h"
 
@@ -397,33 +399,54 @@ class __buffer_impl
     }
 };
 
-template <typename _T>
 struct __sycl_usm_free
 {
-    sycl::queue __q;
+    std::optional<sycl::queue> __q;
 
     void
-    operator()(_T* __memory) const
+    operator()(void* __memory) const
     {
-        sycl::free(__memory, __q.get_context());
+        assert(__q.has_value());
+        sycl::free(__memory, *__q);
     }
 };
 
+// TODO: remove this function once it is no more used in __result_and_scratch_storage
+template <typename _T, sycl::usm::alloc __alloc_t>
+_T*
+__sycl_usm_alloc(const sycl::queue& __q, std::size_t __elements)
+{
+    if (_T* __buf = sycl::malloc<_T>(__elements, __q, __alloc_t))
+        return __buf;
+
+    throw std::bad_alloc();
+}
+
 template <typename _T, sycl::usm::alloc __alloc_t>
-struct __sycl_usm_alloc
+_T*
+__allocate_usm(const sycl::queue& __q, std::size_t __elements)
 {
-    sycl::queue __q;
-
-    _T*
-    operator()(::std::size_t __elements) const
+    static_assert(__alloc_t == sycl::usm::alloc::host || __alloc_t == sycl::usm::alloc::device);
+    _T* __result = nullptr;
+    if constexpr (__alloc_t == sycl::usm::alloc::host)
     {
-        if (auto __buf =
-                static_cast<_T*>(sycl::malloc(sizeof(_T) * __elements, __q.get_device(), __q.get_context(), __alloc_t)))
-            return __buf;
-
-        throw std::bad_alloc();
+#if _ONEDPL_SYCL_L0_EXT_PRESENT
+        // Only use host USM on L0 GPUs. Other devices should use device USM instead to avoid notable slowdown.
+        sycl::device __device = __q.get_device();
+        if (__device.is_gpu() && __device.has(sycl::aspect::usm_host_allocations) &&
+            __device.get_backend() == __dpl_sycl::__level_zero_backend)
+        {
+            __result = sycl::malloc<_T>(__elements, __q, __alloc_t);
+        }
+#endif
     }
-};
+    else
+    {
+        if (__q.get_device().has(sycl::aspect::usm_device_allocations))
+            __result = sycl::malloc<_T>(__elements, __q, __alloc_t);
+    }
+    return __result;
+}
 
 //-----------------------------------------------------------------------
 // type traits for objects granting access to some value objects
@@ -464,46 +487,68 @@ using __repacked_tuple_t = typename __repacked_tuple<T>::type;
 template <typename _ContainerOrIterable>
 using __value_t = typename __internal::__memobj_traits<_ContainerOrIterable>::value_type;
 
-template <typename _Accessor>
-struct __usm_or_buffer_accessor
+//-----------------------------------------------------------------------
+// types to create and use data on a device and return those to the host
+//-----------------------------------------------------------------------
+
+template <typename _T, sycl::access_mode _AccessMode>
+struct __combi_accessor
 {
   private:
-    using _T = std::decay_t<typename _Accessor::value_type>;
-    _Accessor __acc;
+    using __acc_t = sycl::accessor<_T, 1, _AccessMode, __dpl_sycl::__target_device, sycl::access::placeholder::false_t>;
     _T* __ptr = nullptr;
-    bool __usm = false;
-    size_t __offset = 0;
+    __acc_t __acc;
+
+    template <bool __with_offset>
+    __acc_t
+    __make_accessor(bool __fake, sycl::buffer<_T, 1>& __sycl_buf, sycl::handler& __cgh,
+                    const sycl::property_list& __prop_list, std::size_t __sz = 0, std::size_t __offset = 0)
+    {
+        if (__fake)
+        {
+            return __acc_t(
+#if _ONEDPL_SYCL2020_DEFAULT_ACCESSOR_CONSTRUCTOR_BROKEN
+                __sycl_buf, __cgh, __prop_list
+#endif
+            );
+        }
+        if constexpr (__with_offset)
+            return __acc_t(__sycl_buf, __cgh, sycl::range{__sz}, sycl::id{__offset}, __prop_list);
+        else
+            return __acc_t(__sycl_buf, __cgh, __prop_list);
+    }
 
   public:
-    // Buffer accessor
-    __usm_or_buffer_accessor(sycl::handler& __cgh, sycl::buffer<_T, 1>* __sycl_buf,
-                             const sycl::property_list& __prop_list)
-        : __acc(*__sycl_buf, __cgh, __prop_list)
-    {
-    }
-    __usm_or_buffer_accessor(sycl::handler& __cgh, sycl::buffer<_T, 1>* __sycl_buf, size_t __acc_offset,
-                             const sycl::property_list& __prop_list)
-        : __acc(*__sycl_buf, __cgh, __prop_list), __offset(__acc_offset)
-    {
-    }
+    __combi_accessor(sycl::handler& __cgh, sycl::buffer<_T, 1>& __sycl_buf, _T* __usm_buf,
+                     const sycl::property_list& __prop_list)
+        : __ptr(__usm_buf), __acc(__make_accessor<false>(__usm_buf != nullptr, __sycl_buf, __cgh, __prop_list))
+        {}
 
-    // USM pointer
-    __usm_or_buffer_accessor(sycl::handler&, _T* __usm_buf, const sycl::property_list&)
-        : __ptr(__usm_buf), __usm(true)
-    {
-    }
-    __usm_or_buffer_accessor(sycl::handler&, _T* __usm_buf, size_t __ptr_offset, const sycl::property_list&)
-        : __ptr(__usm_buf), __usm(true), __offset(__ptr_offset)
-    {
-    }
+    __combi_accessor(sycl::handler& __cgh, sycl::buffer<_T, 1>& __sycl_buf, _T* __usm_buf, std::size_t __offset,
+                     std::size_t __sz, const sycl::property_list& __prop_list)
+        : __ptr(__usm_buf ? __usm_buf + __offset : nullptr),
+          __acc(__make_accessor<true>(__usm_buf != nullptr, __sycl_buf, __cgh, __prop_list, __sz, __offset))
+        {}
 
-    auto
-    __get_pointer() const // should be cached within a kernel
+    auto // [const] _T*, with constness depending on _AccessMode
+    __data() const // the result should be cached within a kernel
     {
-        return __usm ? __ptr + __offset : &__acc[__offset];
+        return __ptr ? __ptr : &__acc[0];
     }
 };
 
+// The type to exchange information between storage types.
+// Useful for the interoperability during the transition period
+template <typename _T>
+struct __copyable_storage_state
+{
+    std::shared_ptr<_T> __result_buf;
+    std::shared_ptr<_T> __scratch_buf;
+    sycl::buffer<_T, 1> __sycl_buf;
+    std::size_t         __scratch_sz = 0;
+    sycl::usm::alloc    __kind = sycl::usm::alloc::unknown;
+};
+
 // This base class is provided to allow same-typed shared pointer return values from kernels in
 // a `__future` for keeping alive temporary data, while allowing run-time branches to lead to
 // differently typed temporary storage for kernels. Virtual destructor is required to call
@@ -518,6 +563,9 @@ struct __result_and_scratch_storage_base
 template <typename _T, std::size_t _NResults = 1>
 struct __result_and_scratch_storage : __result_and_scratch_storage_base
 {
+    static_assert(sycl::is_device_copyable_v<_T>,
+                  "The type _T must be device copyable to use __result_and_scratch_storage.");
+
   private:
     using __sycl_buffer_t = sycl::buffer<_T, 1>;
 
@@ -525,10 +573,9 @@ struct __result_and_scratch_storage : __result_and_scratch_storage_base
     using __accessor_t =
         sycl::accessor<_T, 1, _AccessMode, __dpl_sycl::__target_device, sycl::access::placeholder::false_t>;
 
-    mutable sycl::queue __q;
     std::shared_ptr<_T> __scratch_buf;
     std::shared_ptr<_T> __result_buf;
-    std::shared_ptr<__sycl_buffer_t> __sycl_buf;
+    mutable __sycl_buffer_t __sycl_buf;
 
     std::size_t __scratch_n;
     bool __use_USM_host;
@@ -536,36 +583,27 @@ struct __result_and_scratch_storage : __result_and_scratch_storage_base
 
     // Only use USM host allocations on L0 GPUs. Other devices show significant slowdowns and will use a device allocation instead.
     bool
-    __use_USM_host_allocations() const
+    __use_USM_host_allocations([[maybe_unused]] const sycl::queue& __q) const
     {
-#if _ONEDPL_SYCL2020_DEFAULT_ACCESSOR_CONSTRUCTOR_PRESENT && _ONEDPL_SYCL_L0_EXT_PRESENT
+        bool __result = false;
+#if _ONEDPL_SYCL_L0_EXT_PRESENT
         auto __device = __q.get_device();
-        if (!__device.is_gpu())
-            return false;
-        if (!__device.has(sycl::aspect::usm_host_allocations))
-            return false;
-        if (__device.get_backend() != __dpl_sycl::__level_zero_backend)
-            return false;
-        return true;
-#else
-        return false;
+        __result = __device.is_gpu() && __device.has(sycl::aspect::usm_host_allocations) &&
+                   __device.get_backend() == __dpl_sycl::__level_zero_backend;
 #endif
+        return __result;
     }
 
     bool
-    __use_USM_allocations() const
+    __use_USM_allocations(const sycl::queue& __q) const
     {
-#if _ONEDPL_SYCL2020_DEFAULT_ACCESSOR_CONSTRUCTOR_PRESENT
         return __q.get_device().has(sycl::aspect::usm_device_allocations);
-#else
-        return false;
-#endif
     }
 
   public:
-    __result_and_scratch_storage(sycl::queue __q_, std::size_t __scratch_n)
-        : __q{__q_}, __scratch_n{__scratch_n}, __use_USM_host{__use_USM_host_allocations()},
-          __supports_USM_device{__use_USM_allocations()}
+    __result_and_scratch_storage(sycl::queue __q, std::size_t __scratch_n)
+        : __sycl_buf{nullptr, sycl::range{0}}, __scratch_n{__scratch_n},
+          __use_USM_host{__use_USM_host_allocations(__q)}, __supports_USM_device{__use_USM_allocations(__q)}
     {
         const std::size_t __total_n = _NResults + __scratch_n;
         // Skip in case this is a dummy container
@@ -577,69 +615,65 @@ struct __result_and_scratch_storage : __result_and_scratch_storage_base
                 if (__scratch_n > 0)
                 {
                     __scratch_buf = std::shared_ptr<_T>(
-                        __internal::__sycl_usm_alloc<_T, sycl::usm::alloc::device>{__q}(__scratch_n),
-                        __internal::__sycl_usm_free<_T>{__q});
+                        __internal::__sycl_usm_alloc<_T, sycl::usm::alloc::device>(__q, __scratch_n),
+                        __internal::__sycl_usm_free{__q});
                 }
                 if constexpr (_NResults > 0)
                 {
                     __result_buf =
-                        std::shared_ptr<_T>(__internal::__sycl_usm_alloc<_T, sycl::usm::alloc::host>{__q}(_NResults),
-                                            __internal::__sycl_usm_free<_T>{__q});
+                        std::shared_ptr<_T>(__internal::__sycl_usm_alloc<_T, sycl::usm::alloc::host>(__q, _NResults),
+                                            __internal::__sycl_usm_free{__q});
                 }
             }
             else if (__supports_USM_device)
             {
                 // If we don't use host memory, malloc only a single unified device allocation
                 __scratch_buf =
-                    std::shared_ptr<_T>(__internal::__sycl_usm_alloc<_T, sycl::usm::alloc::device>{__q}(__total_n),
-                                        __internal::__sycl_usm_free<_T>{__q});
+                    std::shared_ptr<_T>(__internal::__sycl_usm_alloc<_T, sycl::usm::alloc::device>(__q, __total_n),
+                                        __internal::__sycl_usm_free{__q});
             }
             else
             {
                 // If we don't have USM support allocate memory here
-                __sycl_buf = std::make_shared<__sycl_buffer_t>(__sycl_buffer_t(__total_n));
+                __sycl_buf = __sycl_buffer_t(__total_n);
             }
         }
+#if _ONEDPL_SYCL2020_DEFAULT_ACCESSOR_CONSTRUCTOR_BROKEN
+        // A fake buffer to work around problems with accessor construction
+        if (__supports_USM_device)
+            __sycl_buf = __sycl_buffer_t(sycl::range{1});
+#endif
     }
 
+    __result_and_scratch_storage(__copyable_storage_state<_T>&& __transfer)
+        : __scratch_buf(std::move(__transfer.__scratch_buf)), __result_buf(std::move(__transfer.__result_buf)),
+          __sycl_buf(std::move(__transfer.__sycl_buf)), __scratch_n(__transfer.__scratch_sz),
+          __use_USM_host(__transfer.__kind == sycl::usm::alloc::host),
+          __supports_USM_device(__transfer.__kind != sycl::usm::alloc::unknown)
+        {}
+
     template <typename _Acc>
     static auto
-    __get_usm_or_buffer_accessor_ptr(const _Acc& __acc, [[maybe_unused]] std::size_t __scratch_n = 0)
+    __get_usm_or_buffer_accessor_ptr(const _Acc& __acc, std::size_t = 0)
     {
-#if _ONEDPL_SYCL2020_DEFAULT_ACCESSOR_CONSTRUCTOR_PRESENT
-        return __acc.__get_pointer();
-#else
-        return &__acc[__scratch_n];
-#endif
+        return __acc.__data();
     }
 
     template <sycl::access_mode _AccessMode = sycl::access_mode::read_write>
     auto
     __get_result_acc(sycl::handler& __cgh, const sycl::property_list& __prop_list = {}) const
     {
-#if _ONEDPL_SYCL2020_DEFAULT_ACCESSOR_CONSTRUCTOR_PRESENT
         if (__use_USM_host && __supports_USM_device)
-            return __usm_or_buffer_accessor<__accessor_t<_AccessMode>>(__cgh, __result_buf.get(), __prop_list);
-        else if (__supports_USM_device)
-            return __usm_or_buffer_accessor<__accessor_t<_AccessMode>>(__cgh, __scratch_buf.get(), __scratch_n,
-                                                                       __prop_list);
-        return __usm_or_buffer_accessor<__accessor_t<_AccessMode>>(__cgh, __sycl_buf.get(), __scratch_n, __prop_list);
-#else
-        return __accessor_t<_AccessMode>(*__sycl_buf.get(), __cgh, __prop_list);
-#endif
+            return __combi_accessor<_T, _AccessMode>(__cgh, __sycl_buf, __result_buf.get(), __prop_list);
+        return __combi_accessor<_T, _AccessMode>(__cgh, __sycl_buf, __scratch_buf.get(), __scratch_n, _NResults,
+                                                 __prop_list);
     }
 
     template <sycl::access_mode _AccessMode = sycl::access_mode::read_write>
     auto
     __get_scratch_acc(sycl::handler& __cgh, const sycl::property_list& __prop_list = {}) const
     {
-#if _ONEDPL_SYCL2020_DEFAULT_ACCESSOR_CONSTRUCTOR_PRESENT
-        if (__use_USM_host || __supports_USM_device)
-            return __usm_or_buffer_accessor<__accessor_t<_AccessMode>>(__cgh, __scratch_buf.get(), __prop_list);
-        return __usm_or_buffer_accessor<__accessor_t<_AccessMode>>(__cgh, __sycl_buf.get(), __prop_list);
-#else
-        return __accessor_t<_AccessMode>(*__sycl_buf.get(), __cgh, __prop_list);
-#endif
+        return __combi_accessor<_T, _AccessMode>(__cgh, __sycl_buf, __scratch_buf.get(), __prop_list);
     }
 
     _T
@@ -667,13 +701,18 @@ struct __result_and_scratch_storage : __result_and_scratch_storage_base
         }
         else if (__supports_USM_device)
         {
-            _T __tmp;
-            __q.memcpy(&__tmp, __scratch_buf.get() + __scratch_n + _Idx, 1 * sizeof(_T)).wait();
-            return __tmp;
+            auto __q_proxy = std::get_deleter<__internal::__sycl_usm_free>(__scratch_buf);
+            assert(__q_proxy != nullptr && __q_proxy->__q.has_value());
+            // Avoid default constructor for _T. Since _T is device copyable, copy construction
+            // is equivalent to a bitwise copy and we may treat __space.__v as constructed after the memcpy.
+            // There is no need to destroy it afterwards, as the destructor must have no effect.
+            oneapi::dpl::__internal::__lazy_ctor_storage<_T> __space;
+            __q_proxy->__q->memcpy(&__space.__v, __scratch_buf.get() + __scratch_n + _Idx, sizeof(_T)).wait();
+            return __space.__v;
         }
         else
         {
-            return __sycl_buf->get_host_access(sycl::read_only)[__scratch_n + _Idx];
+            return __sycl_buf.get_host_access(sycl::read_only)[__scratch_n + _Idx];
         }
     }
 
@@ -716,6 +755,166 @@ struct __result_and_scratch_storage : __result_and_scratch_storage_base
     }
 };
 
+template <typename _T>
+struct __device_storage
+{
+    std::unique_ptr<_T, __internal::__sycl_usm_free> __usm_buf = nullptr;
+    sycl::buffer<_T, 1> __sycl_buf =
+#if _ONEDPL_SYCL2020_DEFAULT_ACCESSOR_CONSTRUCTOR_BROKEN
+        {sycl::range{1}}; // A non-empty buffer to avoid problems with accessor construction
+#else
+        {nullptr, sycl::range{0}};
+#endif
+
+    __device_storage() = default;
+
+    __device_storage(const sycl::queue& __q, std::size_t __n) { __initialize(__q, __n); }
+
+    template <sycl::access_mode _AccessMode = sycl::access_mode::read_write>
+    auto
+    __get_accessor(sycl::handler& __cgh, const sycl::property_list& __prop_list = {})
+    {
+        return __combi_accessor<_T, _AccessMode>(__cgh, __sycl_buf, __usm_buf.get(), __prop_list);
+    }
+
+  protected:
+    void
+    __initialize(const sycl::queue& __q, std::size_t __n)
+    {
+        assert(__n > 0);
+        _T* __ptr = __internal::__allocate_usm<_T, sycl::usm::alloc::device>(__q, __n);
+        if (__ptr)
+            __usm_buf = std::unique_ptr<_T, __internal::__sycl_usm_free>(__ptr, __internal::__sycl_usm_free{__q});
+        else
+            __sycl_buf = sycl::buffer<_T, 1>(__n);
+    }
+
+    void
+    __copy_n(_T* __dst, _T* __src, std::size_t __n, std::size_t __offset)
+    {
+        // Derived classes are responsible for bound checking
+        if (__src)
+        {
+            std::copy_n(__src, __n, __dst);
+        }
+        else if (__usm_buf)
+        {
+            auto& __q_proxy = __usm_buf.get_deleter();
+            assert(__q_proxy.__q.has_value());
+            __q_proxy.__q->memcpy(__dst, __usm_buf.get() + __offset, __n * sizeof(_T)).wait();
+        }
+        else
+        {
+            std::copy_n(__sycl_buf.get_host_access(sycl::read_only).begin() + __offset, __n, __dst);
+        }
+    }
+};
+
+using oneapi::dpl::__internal::__access_mode_resolver_v;
+
+template <typename _ModeTagT, typename _T>
+auto
+__get_accessor(_ModeTagT, __device_storage<_T>& __st, sycl::handler& __cgh, const sycl::property_list& __prop_list = {})
+{
+    return __st.template __get_accessor<__access_mode_resolver_v<_ModeTagT>>(__cgh, __prop_list);
+}
+
+template <typename _T>
+struct __result_storage : public __device_storage<_T>
+{
+    static_assert(sycl::is_device_copyable_v<_T>, "The type _T must be device copyable to use __result_storage.");
+
+    std::size_t __result_sz = 0;
+    sycl::usm::alloc __kind = sycl::usm::alloc::unknown;
+
+    __result_storage(const sycl::queue& __q, std::size_t __n) : __result_sz(__n)
+    {
+        assert(__result_sz > 0);
+        _T* __ptr = __internal::__allocate_usm<_T, sycl::usm::alloc::host>(__q, __result_sz);
+        if (__ptr)
+        {
+            this->__usm_buf = std::unique_ptr<_T, __internal::__sycl_usm_free>(__ptr, __internal::__sycl_usm_free{__q});
+            __kind = sycl::usm::alloc::host;
+        }
+        else
+        {
+            this->__initialize(__q, __n);
+            __kind = (this->__usm_buf) ? sycl::usm::alloc::device : sycl::usm::alloc::unknown;
+        }
+    }
+
+    // Note: this function assumes a kernel has completed and the result can be transferred to host
+    void
+    __copy_result(_T* __dst, std::size_t __n)
+    {
+        this->__copy_n(__dst, __kind == sycl::usm::alloc::host ? this->__usm_buf.get() : nullptr,
+                       __result_sz < __n ? __result_sz : __n, /*offset*/ 0);
+    }
+};
+
+template <typename _T>
+struct __combined_storage : public __device_storage<_T>
+{
+    static_assert(sycl::is_device_copyable_v<_T>, "The type _T must be device copyable to use __combined_storage.");
+
+    std::unique_ptr<_T, __internal::__sycl_usm_free> __result_buf = nullptr;
+    std::size_t __sz = 0;
+    std::size_t __result_sz = 0;
+    sycl::usm::alloc __kind = sycl::usm::alloc::unknown;
+
+    __combined_storage(const sycl::queue& __q, std::size_t __scratch_n, std::size_t __result_n)
+        : __sz(__scratch_n), __result_sz(__result_n)
+    {
+        assert(__sz > 0 && __result_sz > 0);
+        _T* __ptr = __internal::__allocate_usm<_T, sycl::usm::alloc::host>(__q, __result_sz);
+        if (__ptr)
+        {
+            __result_buf = std::unique_ptr<_T, __internal::__sycl_usm_free>(__ptr, __internal::__sycl_usm_free{__q});
+            this->__initialize(__q, __sz); // a separate scratch buffer
+            __kind = sycl::usm::alloc::host;
+        }
+        else
+        {
+            this->__initialize(__q, __sz + __result_sz); // a combined buffer, starting with scratch
+            __kind = (this->__usm_buf) ? sycl::usm::alloc::device : sycl::usm::alloc::unknown;
+        }
+    }
+
+    // Note: this function assumes a kernel has completed and the result can be transferred to host
+    void
+    __copy_result(_T* __dst, std::size_t __n)
+    {
+        this->__copy_n(__dst, __kind == sycl::usm::alloc::host ? __result_buf.get() : nullptr,
+                       __result_sz < __n ? __result_sz : __n, /*offset*/ __sz);
+    }
+
+    template <typename _ModeTagT>
+    friend auto
+    __get_result_accessor(_ModeTagT, __combined_storage& __st, sycl::handler& __cgh,
+                          const sycl::property_list& __prop_list = {})
+    {
+        if (__st.__kind == sycl::usm::alloc::host)
+        {
+            return __combi_accessor<_T, __access_mode_resolver_v<_ModeTagT>>(
+                __cgh, __st.__sycl_buf, __st.__result_buf.get(), __prop_list);
+        }
+        else
+        {
+            return __combi_accessor<_T, __access_mode_resolver_v<_ModeTagT>>(
+                __cgh, __st.__sycl_buf, __st.__usm_buf.get(), /*offset*/ __st.__sz, __st.__result_sz, __prop_list);
+        }
+    }
+
+    template <typename _Forwarding>
+    friend
+    std::enable_if_t<std::is_same_v<std::decay_t<_Forwarding>, __combined_storage<_T>>, __copyable_storage_state<_T>>
+    __move_state_from(_Forwarding&& __src)
+    {
+        return {std::move(__src.__result_buf), std::move(__src.__usm_buf), std::move(__src.__sycl_buf),
+                __src.__sz, __src.__kind};
+    }
+};
+
 // Tag __async_mode describe a pattern call mode which should be executed asynchronously
 struct __async_mode
 {
@@ -738,7 +937,7 @@ class __future : private std::tuple<_Args...>
     _Event __my_event;
 
     template <typename _T>
-    constexpr _T
+    _T
     __wait_and_get_value(const sycl::buffer<_T>& __buf)
     {
         //according to a contract, returned value is one-element sycl::buffer
@@ -746,13 +945,13 @@ class __future : private std::tuple<_Args...>
     }
 
     template <typename _T, std::size_t _NResults>
-    constexpr auto
+    _T
     __wait_and_get_value(const __result_and_scratch_storage<_T, _NResults>& __storage)
     {
         return __storage.__wait_and_get_value(__my_event);
     }
 
-    constexpr std::pair<std::size_t, std::size_t>
+    std::pair<std::size_t, std::size_t>
     __wait_and_get_value(const std::shared_ptr<__result_and_scratch_storage_base>& __p_storage)
     {
         std::size_t __buf[2] = {0, 0};
@@ -763,7 +962,7 @@ class __future : private std::tuple<_Args...>
     }
 
     template <typename _T>
-    constexpr _T
+    _T
     __wait_and_get_value(const _T& __val)
     {
         wait();
@@ -824,7 +1023,7 @@ class __future : private std::tuple<_Args...>
     //The internal API. There are cases where the implementation specifies return value  "higher" than SYCL backend,
     //where a future is created.
     template <typename _T>
-    auto
+    __future<_Event, _T, _Args...>
     __make_future(_T __t) const
     {
         auto new_val = std::tuple<_T>(__t);
@@ -833,52 +1032,6 @@ class __future : private std::tuple<_Args...>
     }
 };
 
-// Invoke a callable and pass a compile-time integer based on a provided run-time integer.
-// The compile-time integer that will be provided to the callable is defined as the smallest
-// value in the integer_sequence not less than the run-time integer. For example:
-//
-//   __static_monotonic_dispatcher<::std::integer_sequence<::std::uint16_t, 2, 4, 8, 16>::__dispatch(f, 3);
-//
-// will call f<4>(), since 4 is the smallest value in the sequence not less than 3.
-//
-// If there are no values in the sequence less than the run-time integer, the last value in
-// the sequence will be used.
-//
-// Note that the integers provided in the integer_sequence must be monotonically increasing
-template <typename>
-class __static_monotonic_dispatcher;
-
-template <::std::uint16_t _X, ::std::uint16_t... _Xs>
-class __static_monotonic_dispatcher<::std::integer_sequence<::std::uint16_t, _X, _Xs...>>
-{
-    template <::std::uint16_t... _Vals>
-    using _Head = typename ::std::conditional_t<
-        sizeof...(_Vals) != 0,
-        ::std::tuple_element<0, ::std::tuple<::std::integral_constant<::std::uint32_t, _Vals>...>>,
-        ::std::integral_constant<::std::uint32_t, ::std::numeric_limits<::std::uint32_t>::max()>>::type;
-
-    static_assert(_X < _Head<_Xs...>::value, "Sequence must be monotonically increasing");
-
-  public:
-    template <typename _F, typename... _Args>
-    static auto
-    __dispatch(_F&& __f, ::std::uint16_t __x, _Args&&... args)
-    {
-        if constexpr (sizeof...(_Xs) == 0)
-        {
-            return ::std::forward<_F>(__f).template operator()<_X>(::std::forward<_Args>(args)...);
-        }
-        else
-        {
-            if (__x <= _X)
-                return ::std::forward<_F>(__f).template operator()<_X>(::std::forward<_Args>(args)...);
-            else
-                return __static_monotonic_dispatcher<::std::integer_sequence<::std::uint16_t, _Xs...>>::__dispatch(
-                    ::std::forward<_F>(__f), __x, ::std::forward<_Args>(args)...);
-        }
-    }
-};
-
 struct __scalar_load_op
 {
     oneapi::dpl::__internal::__pstl_assign __assigner;
@@ -921,15 +1074,15 @@ struct __scalar_store_transform_op
     // Unary transformations into an output buffer
     template <typename _IdxType1, typename _IdxType2, typename _SourceAcc, typename _DestAcc>
     void
-    operator()(_IdxType1 __idx_source, _IdxType2 __idx_dest, _SourceAcc __source_acc, _DestAcc __dest_acc) const
+    operator()(_IdxType1 __idx_source, _IdxType2 __idx_dest, _SourceAcc&& __source_acc, _DestAcc&& __dest_acc) const
     {
         __transform(__source_acc[__idx_source], __dest_acc[__idx_dest]);
     }
     // Binary transformations into an output buffer
     template <typename _IdxType1, typename _IdxType2, typename _Source1Acc, typename _Source2Acc, typename _DestAcc>
     void
-    operator()(_IdxType1 __idx_source, _IdxType2 __idx_dest, _Source1Acc __source1_acc, _Source2Acc __source2_acc,
-               _DestAcc __dest_acc) const
+    operator()(_IdxType1 __idx_source, _IdxType2 __idx_dest, _Source1Acc&& __source1_acc, _Source2Acc&& __source2_acc,
+               _DestAcc&& __dest_acc) const
     {
         __transform(__source1_acc[__idx_source], __source2_acc[__idx_source], __dest_acc[__idx_dest]);
     }
@@ -997,18 +1150,24 @@ struct __vector_reverse
     static_assert(__vec_size <= 4, "Only vector sizes of 4 or less are supported");
     template <typename _Idx, typename _Array>
     void
-    operator()(/*__is_full*/ std::true_type, const _Idx /*__elements_to_process*/, _Array __array) const
+    operator()(/*__is_full*/ std::true_type, const _Idx /*__elements_to_process*/, _Array&& __array) const
     {
         _ONEDPL_PRAGMA_UNROLL
         for (std::uint8_t __i = 0; __i < __vec_size / 2; ++__i)
-            std::swap(__array[__i], __array[__vec_size - __i - 1]);
+        {
+            using std::swap;
+            swap(__array[__i], __array[__vec_size - __i - 1]);
+        }
     }
     template <typename _Idx, typename _Array>
     void
-    operator()(/*__is_full*/ std::false_type, const _Idx __elements_to_process, _Array __array) const
+    operator()(/*__is_full*/ std::false_type, const _Idx __elements_to_process, _Array&& __array) const
     {
         for (std::uint8_t __i = 0; __i < __elements_to_process / 2; ++__i)
-            std::swap(__array[__i], __array[__elements_to_process - __i - 1]);
+        {
+            using std::swap;
+            swap(__array[__i], __array[__elements_to_process - __i - 1]);
+        }
     }
 };
 
