@@ -122,10 +122,11 @@ struct all_view_fn
     }
 
     template <typename _R>
-    auto
-    operator()(_R&& __r) const -> decltype(std::forward<_R>(__r))
+    decltype(auto)
+    operator()(_R&& __r) const
     {
-        return std::forward<_R>(__r);
+        // Returns __r if it implements operator[] or __subscription_impl_view_simple(__r) otherwise
+        return oneapi::dpl::__ranges::__get_subscription_view(std::forward<_R>(__r));
     }
 };
 
@@ -143,10 +144,11 @@ struct all_host_view_fn
 
     // "No operation" overload for another ranges/views
     template <typename _R>
-    auto
-    operator()(_R&& __r) const -> decltype(::std::forward<_R>(__r))
+    decltype(auto)
+    operator()(_R&& __r) const
     {
-        return ::std::forward<_R>(__r);
+        // Returns __r if it implements operator[] or __subscription_impl_view_simple(__r) otherwise
+        return oneapi::dpl::__ranges::__get_subscription_view(std::forward<_R>(__r));
     }
 };
 #endif
@@ -203,7 +205,7 @@ struct is_sycl_iterator : ::std::false_type
 {
 };
 
-template <oneapi::dpl::access_mode Mode, typename... Types>
+template <sycl::access_mode Mode, typename... Types>
 struct is_sycl_iterator<oneapi::dpl::__internal::sycl_iterator<Mode, Types...>> : ::std::true_type
 {
 };
@@ -251,19 +253,6 @@ using val_t = typename ::std::iterator_traits<_Iter>::value_type;
 
 //range/zip_view/all_view/ variadic utilities
 
-template <typename _Range, typename... _Ranges>
-struct __get_first_range_type
-{
-    using type = _Range;
-};
-
-template <typename _Range, typename... _Ranges>
-constexpr auto
-__get_first_range_size(const _Range& __rng, const _Ranges&...) -> decltype(__rng.size())
-{
-    return __rng.size();
-}
-
 //forward declaration required for _require_access_args
 template <typename _Range, typename... _Ranges>
 void
@@ -320,6 +309,50 @@ __require_access_range(sycl::handler& __cgh, oneapi::dpl::__internal::tuple<_Ran
                                   ::std::make_index_sequence<__num_ranges>());
 }
 
+template <typename T>
+struct __contains_host_pointer : std::false_type
+{
+};
+
+// for std::ranges::ref_view
+#if _ONEDPL_CPP20_RANGES_PRESENT
+template <std::ranges::range Rng>
+struct __contains_host_pointer<std::ranges::ref_view<Rng>> : std::true_type
+{
+};
+#endif // _ONEDPL_CPP20_RANGES_PRESENT
+
+template <typename T>
+using __remove_cv_ref_t = typename std::remove_cv<typename std::remove_reference<T>::type>::type;
+
+template <typename...>
+struct __contains_host_pointer_on_any_layers;
+
+// 1. Checking the top-level view
+// 2. Searching view with host pointer in internal views(layers)
+template <typename _View>
+struct __contains_host_pointer_on_any_layers<_View>
+    : std::disjunction<
+          __contains_host_pointer<__remove_cv_ref_t<_View>>,
+          std::conditional_t<oneapi::dpl::__ranges::is_pipeline_object<_View>::value,
+                             __contains_host_pointer_on_any_layers<
+                                 typename oneapi::dpl::__ranges::pipeline_base_range<_View>::next_layer_view_t>,
+                             std::false_type>>
+{
+};
+
+template <typename... _Ranges>
+struct __contains_host_pointer_on_any_layers<oneapi::dpl::__ranges::zip_view<_Ranges...>>
+    : std::disjunction<__contains_host_pointer_on_any_layers<_Ranges>...>
+{
+};
+
+template <typename _Source, typename _M>
+struct __contains_host_pointer_on_any_layers<oneapi::dpl::__ranges::permutation_view_simple<_Source, _M>>
+    : std::disjunction<__contains_host_pointer_on_any_layers<_Source>, __contains_host_pointer_on_any_layers<_M>>
+{
+};
+
 template <typename _BaseRange>
 void
 __require_access_range(sycl::handler&, _BaseRange&)
@@ -330,6 +363,9 @@ template <typename _Range, typename... _Ranges>
 void
 __require_access(sycl::handler& __cgh, _Range&& __rng, _Ranges&&... __rest)
 {
+    static_assert(!__contains_host_pointer_on_any_layers<std::decay_t<_Range>>::value,
+                  "oneDPL does not support std::ranges::ref_view in SYCL-kernel code");
+
     //getting an access for the all_view based range
     auto base_rng = oneapi::dpl::__ranges::pipeline_base_range<_Range>(::std::forward<_Range>(__rng)).base_range();
 
@@ -370,7 +406,7 @@ struct __range_holder
     }
 };
 
-template <sycl::access::mode AccMode, typename _Iterator>
+template <sycl::access::mode AccMode, typename _Iterator = void> //TODO: _Iterator is not used and should be removed
 struct __get_sycl_range
 {
     __get_sycl_range()
