@@ -64,7 +64,7 @@ DEFINE_TEST(test_find_if)
     {
         TestDataTransfer<UDTKind::eKeys, Size> host_keys(*this, n);
 
-        typedef typename std::iterator_traits<Iterator1>::value_type T1;
+        using T1 = typename std::iterator_traits<Iterator1>::value_type;
 
         std::iota(host_keys.get(), host_keys.get() + n, T1(0));
         host_keys.update_data();
