@@ -23,18 +23,20 @@
 
 #include "support/utils.h"
 
+#include <algorithm> // std::transform
 #include <iostream>
 
-std::int32_t
-main()
-{
 #if _ENABLE_RANGES_TESTING
+template <typename Policy>
+void
+test_impl(Policy&& exec)
+{
     constexpr int max_n = 10;
     int data[max_n] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
     int data2[max_n];
 
-    auto lambda1 = [](auto i) { return i * i; };
-    auto lambda2 = [](auto i) { return i + 200; };
+    auto pred1 = TestUtils::Pow2<int>();
+    auto pred2 = TestUtils::SumWithOp<int>{200};
 
     using namespace oneapi::dpl::experimental::ranges;
 
@@ -43,19 +45,33 @@ main()
         sycl::buffer<int> B(data2, sycl::range<1>(max_n));
 
         auto sv = all_view(A);
-        auto view = views::reverse(sv) | views::transform(lambda1);
+        auto view = views::reverse(sv) | views::transform(pred1);
 
         auto range_res = all_view<int, sycl::access::mode::write>(B);
-        transform(TestUtils::default_dpcpp_policy, view, range_res, lambda2);
+        transform(std::forward<Policy>(exec), view, range_res, pred2);
     }
 
     //check result
     int expected[max_n];
-    ::std::reverse(data, data + max_n);
-    ::std::transform(data, data + max_n, expected, lambda1);
-    ::std::transform(expected, expected + max_n, expected, lambda2);
+    std::reverse(data, data + max_n);
+    std::transform(data, data + max_n, expected, pred1);
+    std::transform(expected, expected + max_n, expected, pred2);
 
     EXPECT_EQ_N(expected, data2, max_n, "wrong effect from transform with sycl ranges");
+}
+#endif // _ENABLE_RANGES_TESTING
+
+std::int32_t
+main()
+{
+#if _ENABLE_RANGES_TESTING
+
+    auto policy = TestUtils::get_dpcpp_test_policy();
+    test_impl(policy);
+
+#if TEST_CHECK_COMPILATION_WITH_DIFF_POLICY_VAL_CATEGORY
+    TestUtils::check_compilation(policy, [](auto&& policy) { test_impl(std::forward<decltype(policy)>(policy)); });
+#endif
 #endif //_ENABLE_RANGES_TESTING
 
     return TestUtils::done(_ENABLE_RANGES_TESTING);
