@@ -19,17 +19,6 @@
 
 #include <cassert>
 
-struct eq
-{
-    eq(int val) : v(val) {}
-    bool
-    operator()(int v2) const
-    {
-        return v == v2;
-    }
-    int v;
-};
-
 void
 kernel_test(sycl::queue& deviceQueue)
 {
@@ -43,11 +32,11 @@ kernel_test(sycl::queue& deviceQueue)
                 int ia[] = {0, 1, 2, 2, 0, 1, 2, 3};
                 const unsigned sa = sizeof(ia) / sizeof(ia[0]);
                 ret_acc[0] &=
-                    (dpl::count_if(input_iterator<const int*>(ia), input_iterator<const int*>(ia + sa), eq(2)) == 3);
+                    (dpl::count_if(input_iterator<const int*>(ia), input_iterator<const int*>(ia + sa), TestUtils::IsEqualTo<int>{2}) == 3);
                 ret_acc[0] &=
-                    (dpl::count_if(input_iterator<const int*>(ia), input_iterator<const int*>(ia + sa), eq(7)) == 0);
+                    (dpl::count_if(input_iterator<const int*>(ia), input_iterator<const int*>(ia + sa), TestUtils::IsEqualTo<int>{7}) == 0);
                 ret_acc[0] &=
-                    (dpl::count_if(input_iterator<const int*>(ia), input_iterator<const int*>(ia), eq(2)) == 0);
+                    (dpl::count_if(input_iterator<const int*>(ia), input_iterator<const int*>(ia), TestUtils::IsEqualTo<int>{2}) == 0);
             });
         });
     }
