@@ -19,14 +19,14 @@ test()
     dpl::complex<T> c;
     assert(c.real() == 0);
     assert(c.imag() == 0);
-    c -= 1.5;
-    assert(c.real() == -1.5);
+    c -= 1.5f;
+    assert(c.real() == -1.5f);
     assert(c.imag() == 0);
-    c -= 1.5;
+    c -= 1.5f;
     assert(c.real() == -3);
     assert(c.imag() == 0);
-    c -= -1.5;
-    assert(c.real() == -1.5);
+    c -= -1.5f;
+    assert(c.real() == -1.5f);
     assert(c.imag() == 0);
 }
 
